//===========================================================================
// Licht und Material
//===========================================================================

//===========================================================================
// includes
//===========================================================================

// GL und GLUT Bibliotheken einbinden (je nach Betriebssystem)
#ifdef WIN32
    #include <windows.h>
    #include <GL/gl.h>
    #include <GL/glut.h>
#else
    #include <GLUT/glut.h>
#endif

#include <math.h>
#include <stdlib.h>

//===========================================================================
// Konstanten
//===========================================================================

const float PI = asin(1.0f)*2.0f; // PI (berechnet)


//===========================================================================
// globale variablen deklarieren und initialisieren
//===========================================================================

int   glutWindow = 0;   // GLUT Fensternummer
float rotation = 0.0f;  // Lichtpositions-Rotation


//===========================================================================
// OpenGL Funktionen
//===========================================================================

void initLight()
{
    // Beleuchtung global einschalten
    glEnable(GL_LIGHTING);

    // Farben der Lichtquelle setzen (r,g,b,a)
    GLfloat ambient[4]  = { 0.1f, 0.0f, 0.0f, 1.0f };
    GLfloat diffuse[4]  = { 0.6f, 0.4f, 0.2f, 1.0f };
    GLfloat specular[4] = { 0.6f, 0.6f, 0.4f, 1.0f };

    glEnable(GL_LIGHT0);
    glLightfv(GL_LIGHT0, GL_AMBIENT,  ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE,  diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, specular);

} // initLight()


void initMaterial()
{
    // Materialfarben setzen (r,g,b,a)
    GLfloat globalAmbient[4] = {0.0, 0.0, 0.0, 1.0};
    GLfloat ambient[4]       = {1.0, 1.0, 1.0, 1.0};
    GLfloat diffuse[4]       = {1.0, 1.0, 1.0, 1.0};
    GLfloat specular[4]      = {1.0, 1.0, 1.0, 1.0};

    // Globale ambiente Beleuchtung komplett herunternehmen
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, globalAmbient);

    // Materialparameter setzen
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
    glMaterialf (GL_FRONT_AND_BACK, GL_SHININESS, 16.0f);

} // initMaterial()


void initGL()
{
    initLight();
    initMaterial();

    // Tiefentest einschalten
    glEnable(GL_DEPTH_TEST);

    // Farben innerhalb von Dreiecken nicht interpolieren
//    glShadeModel(GL_FLAT);

} // initGL()


void transform(unsigned int width, unsigned int height)
{
    // OpenGL Transformationen initialisieren...
    // Ausgabefenstergre setzen
    glViewport(0, 0, (GLint)width, (GLint)height);

    // Verhltnis Breite zu Hhe (aspect) berechnen
    GLdouble aspect = (GLdouble)width / (GLdouble)height;
    
    // Perspektivische Projektion bearbeiten
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    // Kameraperspektive setzen
    GLdouble fieldOfView = atan(0.5/1.0)*180.0/double(PI)*2.0;
    gluPerspective(fieldOfView,  // ffnungswinkel vertikal (field of view)
                   aspect,       // aspect ratio setzen
                   0.001, 10.0); // nahe und ferne Begrenzungsebene setzen

    // Transformationen fr 3D-Koordinaten bearbeiten
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    // Kameraposition setzen
    gluLookAt(0.0, 0.0, 1.5,  // von wo
              0.0, 0.0, 0.0,  // nach wo
              0.0, 1.0, 0.0); // wo ist oben?

} // transform()


//===========================================================================
// Berechnungen (Simulation)
//===========================================================================

void update()
{
    float deltaTime = 1.0f / 60.0f;
    // Rotation weiterbewegen
    rotation += 2.0f * PI * deltaTime / 40.0f;
    // Rotation zwischen 0 und 2*PI begrenzen
    rotation = fmodf(rotation, 2.0f * PI);

} // update()


//===========================================================================
// Zeichnen
//===========================================================================

void render()
{
    // Lichtposition setzen (x,y,z,w)
    GLfloat position[4] = { cos(rotation), 0.5f, sin(rotation), 1.0f };
    glLightfv(GL_LIGHT0, GL_POSITION, position);

    // klein skalierte Teekanne zeichnen
    glutSolidTeapot(0.5);

} // render()


//===========================================================================
// GLUT callbacks
//===========================================================================

void display(void)
{
    // Ausgabefenster in schwarz lschen
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    // Transformationen vor dem Zeichnen setzen
    transform(glutGet(GLUT_WINDOW_WIDTH), glutGet(GLUT_WINDOW_HEIGHT));

    // Simulation aktualisieren
    update();

    // Zeichnen
    render();

    // Hintergrundpuffer, in den gezeichnet wurde, anzeigen
    glutSwapBuffers();

} // display()


void idle(void)
{
    // nochmaliges Zeichnen einleiten
    if (glutWindow != 0) glutPostRedisplay();

} // idle()


void keyboard(unsigned char key, int x, int y)
{
    glutDestroyWindow(glutWindow);
    glutWindow = 0;

    exit(0);

} // keyboard()


//===========================================================================
// Hauptroutine
//===========================================================================

int main(int argc, char* argv[])
{ 
    // GLUT initialisieren
    glutInit(&argc, argv);
    glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);

    // Fenster ffnen
    glutWindow = glutCreateWindow("light");

    // GLUT Funktionen fr Zeichnen, Leerlauf und Tastatur setzen
    glutDisplayFunc(display);
    glutIdleFunc(idle);
    glutKeyboardFunc(keyboard);

     // auf Vollbild gehen und Cursor verbergen
    glutFullScreen();
    glutSetCursor(GLUT_CURSOR_NONE);

    // Alle moeglichen Dinge im Bezug auf OpenGL initialisieren
    // (inklusive Lichtquellen)
    initGL();

    // Hauptschleife des Programms aufrufen
    glutMainLoop();

    // Programm verlassen
    return 0;

} // main()
